'use strict';

const http = require('http');

const { Request, Response } = require('./prototypes');

class Speedlane {
  constructor(port) {
    this._routes = {};
    this._middleware = [];
    
    http.METHODS.forEach(m => this._routes[m] = {});
    
    http.createServer((req, res) => {
      const cbs = [...this._middleware, this._routes[req.method][req.url.split('?')[0]]];
      this._next(cbs, req, res);
    }).listen(port);
  }
  
  on(method, path, cb) {
    this._routes[method][path] = cb;
    return this;
  }
  
  use(cb) {
    this._middleware.unshift(cb);
    return this;
  }

  async _next(cbs, req, res, i = 0) {
    if(!cbs[i]) 
      return;
    
    await cbs[i](req, res, () => this._next(cbs, req, res, ++i));
  }
}

module.exports = { Speedlane, Request, Response };